%{
#include "reach.h"
#include "y.tab.h"
%}

 /* Next two lines are for compatibility with old lex/yacc code */
%option noyywrap
%array

%start		norm comment

LETTER		[a-zA-Z_.]
DIGIT		[0-9]
ID		({LETTER}({LETTER}|{DIGIT})*)

%%
<norm>[ \t\n]		;		/* ignore whitespace */

<norm>[=,()]	 	return(yytext[0]);

<norm>"#"		{
			   BEGIN comment;
			}

<norm>"AND"		return(AND);

<norm>"OR"		return(OR);

<norm>"NAND"		return(NAND);

<norm>"NOR"		return(NOR);

<norm>"NOT"		return(NOT);

<norm>"DFF"		return(DFF);

<norm>"INPUT"		return(INPUT);

<norm>"OUTPUT"		return(OUTPUT);

<norm>{ID}		{
  			    /* store in hash table of wires */
  			    wire w = enter(yytext, yyleng);
		            yylval.wireptr = w;
			    return(ID);
			}

<comment>"\n"		{
			      BEGIN norm;
			}
<comment>(.)		;	/* discard everything inside comment */


<norm>.			{ /* No other rule applies, so it's a bad token */
		   	   fprintf(stderr,"Bad input char `%c' on line %d\n",
			      yytext[0],yylineno);
			}
%%

 /* lexinit - starts the Lexical Analyzer off in the right start condition */
int lexinit()
{
   BEGIN norm;
}

