%{
#include <stdlib.h>
#include "reach.h"

int wire_count; /* to keep track of number of wires in arg lists */

wire *wire_array;
%}

%union		yystacktype
		{
		   wire wireptr;
		   int count;
		   int elementtype;
		}


%start	netlist

%token	INPUT OUTPUT AND OR NAND NOR NOT DFF 
%token	ID

%nonassoc <wireptr> ID

%type <elementtype> element
%type <count> wirelist

%%

netlist		:	netlist definition
		|	definition
		;

element		:	AND	{ $$ = AND;	}
		|	OR 	{ $$ = OR;	}
		|	NAND 	{ $$ = NAND;	}
		|	NOR	{ $$ = NOR;	} 
		|	NOT 	{ $$ = NOT;	}
		|	DFF 	{ $$ = DFF;	}
  		;

definition	:	ID '=' element '(' {wire_count = 0; } wirelist ')'
			{
			  if ($1->type != 0) {
			    pline();
			    fprintf(stdout,
				    "Wire has already been defined: %s",
				    $1->name);
			  }
			  /* error check argument counts */
			  if ((($3 == NOT) || ($3 == DFF)) && (wire_count != 1)) {
			    pline();
			    fprintf(stdout, "Wrong number of arguments to ");
			    print_type($3);
			    fprintf(stdout, ".\n");
			    exit(1);
			  }
			  else if ((wire_count == 0)) {
			    pline();
			    fprintf(stdout, "Warning: no arguments to ");
			    print_type($3);
			    fprintf(stdout, ".\n");
			  }

			  $1->type = $3;
			  $1->fanin_count = wire_count;
			  $1->fanin_array = wire_array;
			  
			  if ($3 == DFF) {
			    present_array[state_var_count] = $1;
			    /* DFF input is next state var, which is
			       an output wire of Next state combinational
			       block. */
			    next_array[state_var_count++] = wire_array[0];
			    wire_array[0]->is_nextstate = 1;
			  }

			  /* for debugging */
			  /* print_wire_def($1); */
			}
		|	INPUT '(' ID ')'
			{
			  if ($3->type != 0) {
			    pline();
			    fprintf(stdout,
				    "Wire has already been defined: %s",
				    $3->name);
			  }
			  $3->type = INPUT;
			  $3->fanin_count = 0;
			  $3->fanin_array = NULL;
			  input_array[input_count++] = $3;
			  /* for debugging */
			  /* printf("INPUT(%s)\n", $3->name); */
			}
		|	OUTPUT '(' ID ')'
			{
			  output_array[output_count++] = $3;
			  $3->is_output = 1;
			}
		;

wirelist	:	ID ',' {wire_count++;} wirelist
			{
			  /* $$ is index of this wire in array */
			  $$ = $4-1;
			  wire_array[$$] = $1;
			}
		|	ID
			{
			  wire_array = (wire *) malloc((++wire_count)*sizeof(wire));
			  $$ = wire_count-1;
			  wire_array[$$] = $1;
			}
		;

%%
