#include <stdio.h>
#include <strings.h>
#include "cudd.h"

/* Size of hash table to hold all wires in circuit. */
#define	TABLESIZE 32768		/* must be a power of 2 */

typedef struct wire_ *wire;

struct wire_ {
  char	*name;			/* Name of wire */
  int 	type;			/* e.g. INPUT, DFF, AND */
  int	is_output;		/* Wire is a primary output. */
  int	is_nextstate;		/* Wire is input to a register. */
  int	fanin_count;		/* number of input wires */
  wire	*fanin_array; 		/* array of input wires */
  int	fanout_count;		/* number of output wires */
  wire	*fanout_array;		/* array of fanout gates
				   (gates are identified with their
				   output wires)*/
  int	fanout_index;		/* for temporary use during fanout
				   computation */
  DdNode *bdd_var;		/* BDD variable for this wire */
  DdNode *bdd;			/* BDD for this wire as function of 
				   primary inputs and present state. */
  int	is_next;		/* Flags to indicate whether gate is on */
  int	is_out;			/* path to state variable or primary output. */
				/* (not mutually exclusive) */
};

/* wirehash.c */

extern wire enter(char *str, int len);
extern int wirevalid(int i);
extern void print_wirehash_table();
extern wire wirehashtable[];

/* reach.c */

extern int input_count;
extern wire input_array[];

extern int output_count;
extern wire output_array[];

extern int state_var_count;		
extern wire present_array[];
extern wire next_array[];

/* LEX, YACC  interface */

extern int yylineno;
extern char yytext[];
extern FILE *yyin;
extern void pline();
extern int yyerror(char *s);
extern void print_type(int t);
extern int lexinit();
extern int yylex(void);
